# Generated by Django 2.0.5 on 2018-06-04 08:16

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Feature',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(default='', max_length=255)),
                ('slug', models.SlugField(default='')),
                ('model_verbose_name', models.CharField(default='', max_length=255)),
                ('limit', models.IntegerField(default=0)),
                ('template', models.CharField(default='', max_length=255)),
            ],
        ),
        migrations.CreateModel(
            name='Query',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('field_verbose_name', models.CharField(default='', max_length=255)),
                ('filter_criteria', models.CharField(choices=[('Equals', 'Equals'), ('Greater than', 'Greater than'), ('Less than', 'Less than'), ('Range', 'Range'), ('Contains', 'Contains')], default='', max_length=255)),
                ('filter_value', models.CharField(default='', max_length=255)),
                ('feature', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, to='qb.Feature')),
            ],
            options={
                'verbose_name_plural': 'Queries',
            },
        ),
    ]



from django.shortcuts import render, redirect, render_to_response
from django.views import generic
from qb.models import *
from qb.features import *

# Create your views here.
class PageView(generic.TemplateView):
    """ Single page view functions """
    template_name = 'qb/index.html'
    def get(self, request, node=False):
        context = self.get_context_data()

        # check enabled features
        ft = Features()
        context["features"] = ft.getft_queryset('sample')

        return render(request, self.template_name, context)





# Generated by Django 2.0.7 on 2018-07-12 14:52

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('qb', '0005_auto_20180704_1840'),
    ]

    operations = [
        migrations.AddField(
            model_name='feature',
            name='order_by',
            field=models.CharField(default='', max_length=255),
        ),
    ]



from django.db import models

# little tweak for Meta attributes
models.options.DEFAULT_NAMES += ('qb',)

# Create your models here.
class Feature(models.Model):
    """ Model class that holds all query instances """
    name = models.CharField(max_length=255, default="")
    slug = models.SlugField(max_length=50, default="")
    model_verbose_name = models.CharField(max_length=255, choices=(), default="")
    limit = models.IntegerField(default=0)
    order_by = models.CharField(max_length=255, choices=(), default="")
    order_asc = models.BooleanField(default=False)
    template = models.CharField(max_length=255, default="")

    def __str__(self):
        return self.name

    class Meta:
        qb = True

class Query(models.Model):
    """ Model for a single query """
    field_verbose_name = models.CharField(max_length=255, default="")    
    FILTERS = (
        ('Equals', 'Equals'),
        ('Greater than', 'Greater than'),
        ('Less than', 'Less than'),
        # ('Range', 'Range'),
        ('Contains', 'Contains'),
    )
    filter_criteria = models.CharField(max_length=255, default="", choices=FILTERS)
    filter_value = models.CharField(max_length=255, default=None, blank=True)
    JOIN_F = (
        ('-', '-'),
        ('AND', 'AND'),
        ('OR', 'OR'),
    )
    join = models.CharField(max_length=255, default="-", choices=JOIN_F)
    feature = models.ForeignKey(Feature, on_delete=models.CASCADE, null=True)

    def __str__(self):
        return self.field_verbose_name
    
    class Meta:
        verbose_name_plural = 'Queries'
        qb = True

class Contact(models.Model):
    name = models.CharField(max_length=255)
    number = models.CharField(max_length=50)

    def __str__(self):
        return self.name



# Generated by Django 2.0.5 on 2018-06-04 10:54

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('qb', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='Contact',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=255)),
                ('number', models.CharField(max_length=50)),
            ],
            options={
                'verbose_name_plural': 'Contact',
            },
        ),
    ]



# Generated by Django 2.0.7 on 2018-07-04 16:40

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('qb', '0004_auto_20180704_1447'),
    ]

    operations = [
        migrations.AlterField(
            model_name='query',
            name='filter_value',
            field=models.CharField(blank=True, default=None, max_length=255),
        ),
    ]



# Generated by Django 2.0.7 on 2018-07-04 12:47

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('qb', '0003_auto_20180607_1638'),
    ]

    operations = [
        migrations.AlterField(
            model_name='query',
            name='join',
            field=models.CharField(choices=[('-', '-'), ('AND', 'AND'), ('OR', 'OR')], default='-', max_length=255),
        ),
    ]



from django.shortcuts import render, redirect, render_to_response
from django.views import generic
from qb.models import *
from django.conf import settings
from django.apps import apps
from django.db.models import Q

# Create your views here.
class Features():

    def getft_queryset(self, slug):
        feature = Feature.objects.get(slug=slug)
        model_name = feature.model_verbose_name
        queries = feature.query_set.all()
        order_field = 'id'
        params = Q()
        for q in queries:
            # print(q.filter_criteria)
            fc = ''
            if q.filter_criteria == 'Greater Than':
                fc = '__gte'
            if q.filter_criteria == 'Less Than':
                fc = '__lte'
            if q.filter_criteria == 'Contains':
                fc = '__contains'
            
            if q.filter_value == 'None':
                fv = None
            else:
                fv = q.filter_value
            
            if feature.order_by != "" and not feature.order_asc:
                order_field = feature.order_by
            elif feature.order_by != "" and feature.order_asc:
                order_field = '-%s' % feature.order_by

            kwargs = {
                "{fbn}{fc}".format(fbn=q.field_verbose_name, fc=fc): fv
            }
            if q.join == "AND":
                params = params & Q(**kwargs)
            else:
                params = params | Q(**kwargs)

        # print (params)       
        model = self.getModelInst(model_name)
        if feature.limit > 0:
            queryset = model.objects.filter(params).order_by(order_field)[:feature.limit]
        else:
            queryset = model.objects.filter(params).order_by(order_field)

        return queryset

    def getModelInst(self, model_name):
        for _app in settings.QB_APPS:
            app_models = apps.get_app_config(_app).get_models()
            res = False
            for model in app_models:
                if not hasattr(model._meta, 'qb'):
                    if model._meta.verbose_name == model_name:
                        res = model
            return res



import os
from setuptools import find_packages, setup

with open(os.path.join(os.path.dirname(__file__), 'README.rst')) as readme:
    README = readme.read()

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name='django-visual-queries',
    version='0.2',
    packages=find_packages(),
    include_package_data=True,
    license='BSD License',  # example license
    description='A simple Django app to visually create DB queries.',
    long_description=README,
    url='http://www.vusakhoza.cherub.co.zw/',
    author='Vusumuzi Khoza',
    author_email='vusakhoza@gmail.com',
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Framework :: Django :: 2.0',  # replace "X.Y" as appropriate
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',  # example license
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
)


# Generated by Django 2.0.5 on 2018-06-07 14:38

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('qb', '0002_contact'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='contact',
            options={},
        ),
        migrations.AddField(
            model_name='query',
            name='join',
            field=models.CharField(choices=[('OR', 'OR'), ('AND', 'AND')], default='', max_length=255),
        ),
        migrations.AlterField(
            model_name='query',
            name='filter_criteria',
            field=models.CharField(choices=[('Equals', 'Equals'), ('Greater than', 'Greater than'), ('Less than', 'Less than'), ('Contains', 'Contains')], default='', max_length=255),
        ),
    ]



from django.contrib import admin
from django.apps import apps
from django.conf import settings

# Register your models here.
from qb.models import *

class QueryInline(admin.TabularInline):
    fields = ('join', 'field_verbose_name', 'filter_criteria', 'filter_value')
    model = Query
    extra = 0

    def get_formset(self, request, obj=None, **kwargs):
        form = super().get_formset(request, obj, **kwargs)
        choicelist = []
        for _app in settings.QB_APPS:
            print(_app)
            app_models = apps.get_app_config(_app).get_models()
            for model in app_models:
                if not hasattr(model._meta, 'qb'):
                    for field in model._meta.fields:
                        field_dets = '%s - %s' % (field.name, model._meta.verbose_name)
                        choicelist.append((field.name, field_dets))
            # obj.model_verbose_name.choices = choicelist
            # print(choicelist)
        Query._meta.get_field('field_verbose_name').choices = choicelist
        # form.fields["field_verbose_name"].choices = choicelist
        return form

class FeatureAdmin(admin.ModelAdmin):
    """ Admin class for Features """
    list_display = ['name', 'slug', 'model_verbose_name', 'template']
    inlines = [
        QueryInline,
    ]
    list_per_page = 10

    def get_form(self, request, obj=None, **kwargs):
        choicelist = []
        fieldlist = []
        for _app in settings.QB_APPS:
            app_models = apps.get_app_config(_app).get_models()
            for model in app_models:
                if not hasattr(model._meta, 'qb'):
                    choicelist.append((model._meta.verbose_name , model._meta.verbose_name))

                    # update field names for order functionality
                    for field in model._meta.fields:
                        field_dets = '%s - %s' % (field.name, model._meta.verbose_name)
                        fieldlist.append((field.name, field_dets))
            # obj.model_verbose_name.choices = choicelist
            # print(choicelist)
        Feature._meta.get_field('model_verbose_name').choices = choicelist
        Feature._meta.get_field('order_by').choices = fieldlist

        return super(FeatureAdmin, self).get_form(request, obj, **kwargs)

admin.site.register(Feature, FeatureAdmin)
# admin.site.register(Contact)
